import argparse
import onnx
import aidge_core

import numpy as np

from typing import Dict
from pathlib import Path
from aidge_onnx import convert_onnx_to_aidge, run_ort_outputwise_benchmark
from aidge_onnx.utils import set_every_out_as_graph_out
from aidge_core.benchmark.output_wise_comparison import (
    run_aidge_outputwise_benchmark,
    compare_outputs,
    render_results_table,
    save_comparison_differences
)

## Utils function
def generate_random_inputs_from_onnx(model: onnx.ModelProto) -> Dict[str, np.ndarray]:
    inputs = {}
    initializer_names = [x.name for x in model.graph.initializer]
    for inp in model.graph.input:
        # Skip initializers
        if inp.name in initializer_names: continue
        shape = []
        for dim in inp.type.tensor_type.shape.dim:
            if dim.dim_value > 0:
                shape.append(dim.dim_value)
            else:
                shape.append(1)  # Use 1 as a fallback for unknown dimensions
        dtype = inp.type.tensor_type.elem_type
        np_dtype = onnx.mapping.TENSOR_TYPE_TO_NP_TYPE.get(dtype, np.float32)
        rand_input = np.random.rand(*shape).astype(np_dtype)
        inputs[inp.name] = rand_input
    return inputs

def compare_ort_aidge():
    parser = argparse.ArgumentParser(description="Compare ONNXRuntime with Aidge")
    parser.add_argument("model_path", type=str, help="Path to the ONNX model")
    parser.add_argument("--log-dir", type=str, default=None, help="Directory to save result logs (optional)")
    parser.add_argument("--atol", type=float, default=1e-4, help="TODO")
    parser.add_argument("--rtol", type=float, default=1e-3, help="TODO")

    args = parser.parse_args()

    model_path = args.model_path
    aidge_core.Log.set_console_level(aidge_core.Level.Fatal)




    # Load model
    model = onnx.load(model_path)
    set_every_out_as_graph_out(model)
    # Create Aidge model
    graph_view = convert_onnx_to_aidge(model)

    # Prepare inputs
    input_data = generate_random_inputs_from_onnx(model)


    # Run inferences
    onnx_outputs = run_ort_outputwise_benchmark(model, input_data)
    aidge_outputs = run_aidge_outputwise_benchmark(graph_view, input_data)

    # Compare and render
    results = compare_outputs(onnx_outputs, aidge_outputs, atol=args.atol, rtol=args.rtol)
    render_results_table(results, "ONNX Runtime", "Aidge", Path(model_path).stem)

    if args.log_dir:
        save_comparison_differences(results, output_dir=args.log_dir)

if __name__ == "__main__":
    compare_ort_aidge()
